@@ -36,14 +36,38 @@ from typing import Dict, Tuple, Union, List, Iterator, Any, NoReturn
 """
 
 """
-from os import cpu_count
 from random import random
 from time import sleep
+from aitool import pool_starmap, multi_map, exe_time
+import functools
+from collections.abc import Iterable
+from os import cpu_count
+from random import random
+from time import sleep, time
+from typing import Iterator, Callable, NoReturn
+
 import multiprocess as mp
-from aitool import pool_map, pool_starmap, multi_map, exe_time
 
 
-SLEEP_TIME = [random() for _ in range(10000)]
+def pool_map(
+        func: Callable,
+        conditions: Iterable,
+        processes: int = cpu_count(),
+        initializer=None,
+        initargs=(),
+        maxtasksperchild=None,
+):
+    # 基于pool.map实现
+    with mp.Pool(
+            processes=processes,
+            initializer=initializer,
+            initargs=initargs,
+            maxtasksperchild=maxtasksperchild,
+    ) as p:
+        for result in p.imap(func, conditions):
+            yield result
+
+SLEEP_TIME = [random() for _ in range(20)]
 
 
 def toy(x):
@@ -62,29 +86,26 @@ def test_sequence():
 
 
 @exe_time(print_time=True)
-def test_use_multi(ordered=False):
-    functions = list(get_functions(toy, SLEEP_TIME))
-    data = [result for result in multi(functions, ordered=ordered)]
+def test_pool_map():
+    data = [result for result in pool_map(toy, SLEEP_TIME)]
     do_something_in_parent_process(data)
 
 
 @exe_time(print_time=True)
-def test_use_multi(ordered=False):
-    functions = list(get_functions(toy, SLEEP_TIME))
-    data = [result for result in multi(functions, ordered=ordered)]
+def test_pool_starmap():
+    data = [result for result in pool_starmap(toy, [[_] for _ in SLEEP_TIME])]
     do_something_in_parent_process(data)
 
 
-
 @exe_time(print_time=True)
-def test_use_multi(ordered=False):
-    functions = list(get_functions(toy, SLEEP_TIME))
-    data = [result for result in multi(functions, ordered=ordered)]
+def test_multi_map():
+    data = [result for result in multi_map(toy, SLEEP_TIME)]
     do_something_in_parent_process(data)
 
 
 print(sum(SLEEP_TIME))
-test_use_pool()
-test_use_multi()
-test_use_multi(ordered=True)
+print(SLEEP_TIME)
+test_pool_map()
+test_pool_starmap()
+test_multi_map()
 test_sequence()
